;
;   ͻ
;     FILE NAME: PSDEMO.ASM             SCOPE:  This file is used to     
;   Ĵ  demonstrate how PS - simulator   
;     LINGUAGE:  16C54 Assembler        is working.                      
;   Ĵ                                   
;     VERSION: 1.0  REVISION: 1.0                                       
;   Ķ
;     DATE: 10/05/94                    AUTHOR: Sam Lowry (c)            
;   ͼ
;
;
;             This program is used as demonstration of PS.EXE
;
;   Program simulates code transmitter. It can be used, for example, as
;   remote controller for garden-gate of garage door. It will transmitt
;   a train of impulses following protocol:
;
;
;   Code is 2 bytes long word. Each byte has data 8 bits. Before to start
;   transmission, it will be transmitted start pulse, and at the end
;   stop pulse. Pause between two words is 10 To.
;
;   To = 100 S
;
;   Logical "1" is represented as pulse + 2 x To
;   Logical "0" is represented as pulse + 1 x To
;   Start pulse is represented as pulse + 3 x To
;   Stop  pulse is represented as pulse + 3 x To
;
;   Signal is PWM with 10S long pulse and 20S silence (33% duty cycle).
;
;   Signal is 10 pulses.
;    ڿ ڿ ڿ ڿ ڿ ڿ ڿ ڿ ڿ ڿ                  ڿ
;                               
;                               
;                               
;          
;                                                 
;    <Signal><Pause>
;                                                 
;
;   All timings are calculated for 4 MHz XT PIC 16C54.
;

; ************** DEFINITION OF THE CONSTANTS AND VARIABLES *******************

              CUST_ADDR EQU    B'01010010'  ; Custom address
              CUST_DATA EQU    B'11101101'  ; Custom data

              Same      EQU     .1

              RTCC      EQU     0x001       ; Real Time Clock Counter F1
              PC        EQU     0x002       ; Program Counter
              SWR       EQU     0x003       ; SWR
              PortA     EQU     0x005       ; Internal addresses for I/O PortA

              TXOut     EQU     .0          ; Bit 0 is output
              Carry     EQU     .0


              BitNum    EQU     0x1B
              Cycle     EQU     0x1C
              WaitLen   EQU     0x1D
              TXBuffer  EQU     0x1E
              Counter   EQU     0x1F        ; RAM file register

; ************************** FUNCTIONS **************************************

              ORG       0x000



; ********************

SEND_PULSE    MOVLW     .10                 ; Make 10 loops
              MOVWF     Counter

SEND_LOOP     BSF       PortA,TXOut         ; Make a 10 S Strobe
              NOP
              NOP
              NOP
              NOP
              NOP
              NOP
              NOP
              NOP
              NOP
              BCF       PortA,TXOut         ; SWITCH Off output

WAIT_20       MOVLW     .5                  ; Wait for 20 S
              MOVWF     Cycle
WAIT_20LOOP   DECFSZ    Cycle,Same
              GOTO      WAIT_20LOOP
              NOP

              DECFSZ    Counter,Same
              GOTO      SEND_LOOP           ; And loop while Counter>0
              RETLW     0


; ********************

WAIT_To       MOVLW     .31                 ; Wait for 100S x WaitLen
              MOVWF     Cycle
WTo_LOOP      DECFSZ    Cycle,Same          ; 31 x 3 S = 93 S
              GOTO      WTo_LOOP
              DECFSZ    WaitLen             ; 1 S
              GOTO      WAIT_To             ; 2 S
              RETLW     0

; ********************

SEND_BYTE     MOVLW     .8                  ; Send 8 bits
              MOVWF     BitNum
BYTE_LOOP     CALL      SEND_PULSE
              BTFSS     TXBuffer,0          ; Test if Bit is 0
              GOTO      BIT_0
BIT_1         MOVLW     .2                  ; Bit = 0 => Wait 2 x To
              GOTO      CONTINUE
BIT_0         MOVLW     .1                  ; Bit = 1 => Wait 1 x To
              GOTO      CONTINUE

CONTINUE      MOVWF     WaitLen
              CALL      WAIT_To

              BCF       SWR,Carry
              RRF       TXBuffer,Same

              DECFSZ    BitNum,Same
              GOTO      BYTE_LOOP

              RETLW     0


; ********************************* MAIN ************************************

              ORG       0x100

MainStart     MOVLW     B'00001111'         ; Prepare port A
              MOVWF     PortA
              MOVLW     B'00001110'         ; Bit 0 is transmitter
              TRIS      PortA

MainLoop      CALL     SEND_PULSE           ; Start pulse

              MOVLW    .3                   ; Wait 3 x To
              MOVWF    WaitLen
              CALL     WAIT_To

              MOVLW    CUST_ADDR            ; Send custom address
              MOVWF    TXBuffer
              CALL     SEND_BYTE

              MOVLW    CUST_DATA            ; Send custom data
              MOVWF    TXBuffer
              CALL     SEND_BYTE

              CALL     SEND_PULSE           ;  ¿
                                            ;   
              MOVLW    .3                   ;   
              MOVWF    WaitLen              ;    Stop pulse
              CALL     WAIT_To              ;   
                                            ;   
              CALL     SEND_PULSE           ;  

              MOVLW    .10                  ;  ¿
              MOVWF    WaitLen              ;    Pause between two codes
              CALL     WAIT_To              ;  

              GOTO     MainLoop

; ===========================================================================

              ORG      0x1FF
              GOTO     MainStart
              END
